@@ -52,6 +52,17 @@ extern Bool			g_fNoHelpMessageBox;
 extern Bool			g_fSoftwareCursor;
 extern Bool			g_fSilentDupError;
 
+/* globals required by callback function for monitor information */
+struct GetMonitorInfoData {
+    int  requestedMonitor;
+    int  monitorNum;
+    Bool bUserSpecifiedMonitor;
+    Bool bMonitorSpecifiedExists;
+    int  monitorOffsetX;
+    int  monitorOffsetY;
+    int  monitorHeight;
+    int  monitorWidth;
+};
 
 /*
  * Function prototypes
@@ -70,6 +81,7 @@ void OsVendorVErrorF (const char *pszFormat, va_list va_args);
 void
 winInitializeDefaultScreens (void);
 
+BOOL CALLBACK getMonitorInfo(HMONITOR hMonitor, HDC hdc, LPRECT rect, LPARAM data);
 
 /*
  * Process arguments on the command line
@@ -252,6 +264,7 @@ ddxProcessArgument (int argc, char *argv[], int i)
       int		iArgsProcessed = 1;
       int		nScreenNum;
       int		iWidth, iHeight, iX, iY;
+      int		iMonitor;
 
 #if CYGDEBUG
       winDebug ("ddxProcessArgument - screen - argc: %d i: %d\n",
@@ -276,8 +289,40 @@ ddxProcessArgument (int argc, char *argv[], int i)
 	  return 0;
         }
 
+	  /* look for @m where m is monitor number */
+	  if (i + 2 < argc
+		  && 1 == sscanf(argv[i + 2], "@%d", (int *) &iMonitor)) 
+      {
+        struct GetMonitorInfoData data;
+        memset(&data, 0, sizeof(data));
+        data.requestedMonitor = iMonitor;
+		EnumDisplayMonitors(NULL, NULL, getMonitorInfo, (LPARAM) &data);
+		if (data.bMonitorSpecifiedExists == TRUE) 
+        {
+		  winErrorFVerb(2, "ddxProcessArgument - screen - Found Valid ``@Monitor'' = %d arg\n", iMonitor);
+		  iArgsProcessed = 3;
+		  g_ScreenInfo[nScreenNum].fUserGaveHeightAndWidth = FALSE;
+		  g_ScreenInfo[nScreenNum].fUserGavePosition = TRUE;
+		  g_ScreenInfo[nScreenNum].dwWidth = data.monitorWidth;
+		  g_ScreenInfo[nScreenNum].dwHeight = data.monitorHeight;
+		  g_ScreenInfo[nScreenNum].dwUserWidth = data.monitorWidth;
+		  g_ScreenInfo[nScreenNum].dwUserHeight = data.monitorHeight;
+		  g_ScreenInfo[nScreenNum].dwInitialX = data.monitorOffsetX;
+		  g_ScreenInfo[nScreenNum].dwInitialY = data.monitorOffsetY;
+		}
+		else 
+        {
+		  /* monitor does not exist, error out */
+		  ErrorF ("ddxProcessArgument - screen - Invalid monitor number %d\n",
+				  iMonitor);
+		  UseMsg ();
+		  exit (0);
+		  return 0;
+		}
+	  }
+
       /* Look for 'WxD' or 'W D' */
-      if (i + 2 < argc
+      else if (i + 2 < argc
 	  && 2 == sscanf (argv[i + 2], "%dx%d",
 			  (int *) &iWidth,
 			  (int *) &iHeight))
@@ -298,6 +343,58 @@ ddxProcessArgument (int argc, char *argv[], int i)
 	    g_ScreenInfo[nScreenNum].fUserGavePosition = TRUE;
 	    g_ScreenInfo[nScreenNum].dwInitialX = iX;
 	    g_ScreenInfo[nScreenNum].dwInitialY = iY;
+
+		/* look for WxD+X+Y@m where m is monitor number. take X,Y to be offsets from monitor's root position */
+		if (1 == sscanf (argv[i + 2], "%*dx%*d+%*d+%*d@%d",
+						 (int *) &iMonitor)) 
+        {
+          struct GetMonitorInfoData data;
+          memset(&data, 0, sizeof(data));
+          data.requestedMonitor = iMonitor;
+		  EnumDisplayMonitors(NULL, NULL, getMonitorInfo, (LPARAM) &data);
+		  if (data.bMonitorSpecifiedExists == TRUE) 
+          {
+			g_ScreenInfo[nScreenNum].dwInitialX += data.monitorOffsetX;
+			g_ScreenInfo[nScreenNum].dwInitialY += data.monitorOffsetY;
+		  }
+		  else 
+          {
+			/* monitor does not exist, error out */
+			ErrorF ("ddxProcessArgument - screen - Invalid monitor number %d\n",
+					iMonitor);
+			UseMsg ();
+			exit (0);
+			return 0;
+		  }
+
+		}
+	  }
+
+	  /* look for WxD@m where m is monitor number */
+	  else if (1 == sscanf(argv[i + 2], "%*dx%*d@%d",
+						   (int *) &iMonitor)) 
+      {
+        struct GetMonitorInfoData data;
+        memset(&data, 0, sizeof(data));
+        data.requestedMonitor = iMonitor;
+		EnumDisplayMonitors(NULL, NULL, getMonitorInfo, (LPARAM) &data);
+		if (data.bMonitorSpecifiedExists == TRUE) 
+        {
+		  winErrorFVerb (2, "ddxProcessArgument - screen - Found Valid ``@Monitor'' = %d arg\n", iMonitor);
+		  g_ScreenInfo[nScreenNum].fUserGavePosition = TRUE;
+		  g_ScreenInfo[nScreenNum].dwInitialX = data.monitorOffsetX;
+		  g_ScreenInfo[nScreenNum].dwInitialY = data.monitorOffsetY;
+		}
+		else 
+        {
+		  /* monitor does not exist, error out */
+		  ErrorF ("ddxProcessArgument - screen - Invalid monitor number %d\n",
+				  iMonitor);
+		  UseMsg ();
+		  exit (0);
+		  return 0;
+		}
+
 	  }
 	}
       else if (i + 3 < argc
@@ -1332,3 +1429,24 @@ winLogVersionInfo (void)
   ErrorF ("Release: %s\n\n", VERSION_STRING);
   ErrorF ("Contact: %s\n\n", VENDOR_CONTACT);
 }
+
+/*
+ * getMonitorInfo - callback function used to return information from the enumeration of monitors attached
+ */
+
+BOOL CALLBACK getMonitorInfo(HMONITOR hMonitor, HDC hdc, LPRECT rect, LPARAM _data) 
+{
+  struct GetMonitorInfoData* data = (struct GetMonitorInfoData*)_data;
+  // only get data for monitor number specified in <data>
+  data->monitorNum++;
+  if (data->monitorNum == data->requestedMonitor) 
+  {
+	data->bMonitorSpecifiedExists = TRUE;
+	data->monitorOffsetX = rect->left;
+	data->monitorOffsetY = rect->top;
+	data->monitorHeight  = rect->bottom - rect->top;
+	data->monitorWidth   = rect->right  - rect->left;
+    return FALSE;
+  }
+  return TRUE;
+}
